@@ -33,6 +33,7 @@ import org.apache.flink.kubernetes.operator.reconciler.deployment.ReconcilerFact
 import org.apache.flink.kubernetes.operator.service.FlinkResourceContextFactory;
 import org.apache.flink.kubernetes.operator.utils.EventRecorder;
 import org.apache.flink.kubernetes.operator.utils.EventSourceUtils;
+import org.apache.flink.kubernetes.operator.utils.ExceptionUtils;
 import org.apache.flink.kubernetes.operator.utils.KubernetesClientUtils;
 import org.apache.flink.kubernetes.operator.utils.StatusRecorder;
 import org.apache.flink.kubernetes.operator.utils.ValidatorUtils;
@@ -154,17 +155,15 @@ public class FlinkDeploymentController
             statusRecorder.patchAndCacheStatus(flinkApp, ctx.getKubernetesClient());
             reconcilerFactory.getOrCreate(flinkApp).reconcile(ctx);
         } catch (UpgradeFailureException ufe) {
-            handleUpgradeFailure(ctx, ufe);
+            ReconciliationUtils.updateForReconciliationError(ctx, ufe);
+            triggerErrorEvent(ctx, ufe, ufe.getReason());
         } catch (DeploymentFailedException dfe) {
-            handleDeploymentFailed(ctx, dfe);
+            flinkApp.getStatus().setJobManagerDeploymentStatus(JobManagerDeploymentStatus.ERROR);
+            flinkApp.getStatus().getJobStatus().setState(JobStatus.RECONCILING);
+            ReconciliationUtils.updateForReconciliationError(ctx, dfe);
+            triggerErrorEvent(ctx, dfe, dfe.getReason());
         } catch (Exception e) {
-            eventRecorder.triggerEvent(
-                    flinkApp,
-                    EventRecorder.Type.Warning,
-                    "ClusterDeploymentException",
-                    e.getMessage(),
-                    EventRecorder.Component.JobManagerDeployment,
-                    josdkContext.getClient());
+            triggerErrorEvent(ctx, e, EventRecorder.Reason.Error.name());
             throw new ReconciliationException(e);
         }
 
@@ -174,32 +173,13 @@ public class FlinkDeploymentController
                 ctx.getOperatorConfig(), flinkApp, previousDeployment, true);
     }
 
-    private void handleDeploymentFailed(
-            FlinkResourceContext<FlinkDeployment> ctx, DeploymentFailedException dfe) {
-        var flinkApp = ctx.getResource();
-        LOG.error("Flink Deployment failed", dfe);
-        flinkApp.getStatus().setJobManagerDeploymentStatus(JobManagerDeploymentStatus.ERROR);
-        flinkApp.getStatus().getJobStatus().setState(JobStatus.RECONCILING);
-        ReconciliationUtils.updateForReconciliationError(ctx, dfe);
+    private void triggerErrorEvent(
+            FlinkResourceContext<FlinkDeployment> ctx, Exception e, String reason) {
         eventRecorder.triggerEvent(
-                flinkApp,
+                ctx.getResource(),
                 EventRecorder.Type.Warning,
-                dfe.getReason(),
-                dfe.getMessage(),
-                EventRecorder.Component.JobManagerDeployment,
-                ctx.getKubernetesClient());
-    }
-
-    private void handleUpgradeFailure(
-            FlinkResourceContext<FlinkDeployment> ctx, UpgradeFailureException ufe) {
-        LOG.error("Error while upgrading Flink Deployment", ufe);
-        var flinkApp = ctx.getResource();
-        ReconciliationUtils.updateForReconciliationError(ctx, ufe);
-        eventRecorder.triggerEvent(
-                flinkApp,
-                EventRecorder.Type.Warning,
-                ufe.getReason(),
-                ufe.getMessage(),
+                reason,
+                ExceptionUtils.getExceptionMessage(e),
                 EventRecorder.Component.JobManagerDeployment,
                 ctx.getKubernetesClient());
     }
