@@ -211,6 +211,19 @@ public class ReconciliationUtils {
         reconciliationStatus.setReconciliationTimestamp(System.currentTimeMillis());
     }
 
+    public static <SPEC extends AbstractFlinkSpec> void updateLastReconciledAutoscalerResetNonce(
+            AbstractFlinkResource<SPEC, ?> target) {
+        var spec = target.getSpec();
+        var reconciliationStatus = target.getStatus().getReconciliationStatus();
+        var lastReconciledSpec = reconciliationStatus.deserializeLastReconciledSpec();
+
+        lastReconciledSpec
+                .getJob()
+                .setAutoscalerResetNonce(spec.getJob().getAutoscalerResetNonce());
+        reconciliationStatus.serializeAndSetLastReconciledSpec(lastReconciledSpec, target);
+        reconciliationStatus.setReconciliationTimestamp(System.currentTimeMillis());
+    }
+
     private static void updateLastReconciledJobSpec(
             JobSpec lastReconciledJobSpec, JobSpec jobSpec, SnapshotType snapshotType) {
         switch (snapshotType) {
@@ -546,11 +559,6 @@ public class ReconciliationUtils {
         var lastSpecWithMeta = reconciliationStatus.deserializeLastReconciledSpecWithMeta();
         var newMeta = ReconciliationMetadata.from(resource);
 
-        if (newMeta.equals(lastSpecWithMeta.getMeta())) {
-            // Nothing to update
-            return;
-        }
-
         reconciliationStatus.setLastReconciledSpec(
                 SpecUtils.writeSpecWithMeta(lastSpecWithMeta.getSpec(), newMeta));
         resource.getStatus().setObservedGeneration(resource.getMetadata().getGeneration());
